@@ -1,4 +1,4 @@
-'''
+"""
 Module defining the `Network` object, the basis of all simulation runs.
 
 Preferences
@@ -6,7 +6,7 @@ Preferences
 
 .. document_brian_prefs:: core.network
 
-'''
+"""
 
 import os
 import sys
@@ -43,16 +43,16 @@ prefs.register_preferences('core.network', 'Network preferences',
                                         'resets',
                                         'end',
                                         ],
-                               docs='''
+                               docs="""
                                Default schedule used for networks that
                                don't specify a schedule.
-                               '''
+                               """
                            )
                            )
 
 
 def _format_time(time_in_s):
-    '''
+    """
     Helper function to format time in seconds, minutes, hours, days, depending
     on the magnitude.
 
@@ -68,7 +68,7 @@ def _format_time(time_in_s):
     >>> _format_time(.5)
     '< 1s'
 
-    '''
+    """
     divisors = [24*60*60, 60*60, 60, 1]
     letters = ['d', 'h', 'm', 's']
     remaining = time_in_s
@@ -79,7 +79,7 @@ def _format_time(time_in_s):
         if time_to_represent > 0 or len(text):
             if len(text):
                 text += ' '
-            text += '%d%s' % (time_to_represent, letter)
+            text += f'{int(time_to_represent)}{letter}'
 
     # less than one second
     if len(text) == 0:
@@ -89,30 +89,31 @@ def _format_time(time_in_s):
 
 
 class TextReport(object):
-    '''
+    """
     Helper object to report simulation progress in `Network.run`.
 
     Parameters
     ----------
     stream : file
         The stream to write to, commonly `sys.stdout` or `sys.stderr`.
-    '''
+    """
     def __init__(self, stream):
         self.stream = stream
 
     def __call__(self, elapsed, completed, start, duration):
         if completed == 0.0:
-            self.stream.write(('Starting simulation at t=%s for a duration of '
-                               '%s\n') % (start, duration))
+            self.stream.write(f"Starting simulation at t={start} for a duration of "
+                              f"{duration}\n")
         else:
-            report_msg = ('{t} ({percent}%) simulated in '
-                          '{real_t}').format(t=str(completed*duration),
-                                             percent=int(completed*100.),
-                                             real_t=_format_time(float(elapsed)))
+            t = str(completed * duration)
+            percent = int(completed * 100.)
+            real_t = _format_time(float(elapsed))
+            report_msg = (f"{t} ({percent}%) simulated in "
+                          f"{real_t}")
             if completed < 1.0:
                 remaining = int(round((1-completed)/completed*float(elapsed)))
-                remaining_msg = (', estimated {remaining} '
-                                 'remaining.\n').format(remaining=_format_time(remaining))
+                remaining_msg = (f", estimated {_format_time(remaining)} "
+                                 f"remaining.\n")
             else:
                 remaining_msg = '\n'
 
@@ -139,7 +140,7 @@ def _format_table(header, values, cell_formats):
 
 
 class SchedulingSummary(object):
-    '''
+    """
     Object representing the schedule that is used to simulate the objects in a
     network. Objects of this type are returned by `scheduling_summary`, they
     should not be created manually by the user.
@@ -148,7 +149,7 @@ class SchedulingSummary(object):
     ----------
     objects : list of `BrianObject`
         The sorted list of objects that are simulated by the network.
-    '''
+    """
     def __init__(self, objects):
         # Map each dt to a rank (i.e. smallest dt=0, second smallest=1, etc.)
         self.dts = dict((dt, rank) for rank, dt in
@@ -182,8 +183,8 @@ class SchedulingSummary(object):
 
     def __repr__(self):
         return _format_table(['object', 'part of', 'Clock dt', 'when', 'order', 'active'],
-                             [['{} ({})'.format(entry.name, entry.type),
-                               '{} ({})'.format(entry.owner_name, entry.owner_type)
+                             [[f'{entry.name} ({entry.type})',
+                               f'{entry.owner_name} ({entry.owner_type})'
                                if entry.owner_name is not None else '--',
                                '{} (every {})'.format(str(entry.dt),
                                                      'step' if self.steps[float(entry.dt)] == 1
@@ -194,7 +195,7 @@ class SchedulingSummary(object):
                              ['{:<{}}', '{:<{}}', '{:<{}}', '{:<{}}', '{:{}d}', '{:^{}}'])
 
     def _repr_html_(self):
-        rows = ['''\
+        rows = ["""\
         <tr>
             <td style="text-align: left;">{}</td>
             <td style="text-align: left;">{}</td>
@@ -203,17 +204,17 @@ class SchedulingSummary(object):
             <td style="text-align: right;">{}</td>
             <td style="text-align: center;">{}</td>
         </tr>
-        '''.format('<b>{}</b> (<em>{}</em>)'.format(entry.name, entry.type),
+        """.format('<b>{}</b> (<em>{}</em>)'.format(entry.name, entry.type),
                    '{} (<em>{}</em>)'.format(entry.owner_name, entry.owner_type)
                    if entry.owner_name is not None else '&ndash;',
                    '{} (every {})'.format(str(entry.dt),
                                           'step' if self.steps[float(entry.dt)] == 1
-                                          else '{} steps'.format(self.steps[float(entry.dt)])),
+                                          else f'{self.steps[float(entry.dt)]} steps'),
                    entry.when,
                    entry.order,
                    'yes' if entry.active else 'no')
                 for entry in self.entries]
-        html_code = '''
+        html_code = """
         <table>
         <thead>
         <tr>
@@ -229,12 +230,12 @@ class SchedulingSummary(object):
 {rows}
         </tbody>
         </table>
-        '''.format(rows='\n'.join(rows))
+        """.format(rows='\n'.join(rows))
         return html_code
 
 
 def _check_multiple_summed_updaters(objects):
-    '''
+    """
     Helper function that checks whether multiple `SummedVariableUpdater` target
     the same target variable. Raises a `NotImplementedError` if this is the
     case (and problematic, i.e. not when using non-overlapping subgroups).
@@ -243,7 +244,7 @@ def _check_multiple_summed_updaters(objects):
     ----------
     objects : list of `BrianObject`
         The list of objects in the network.
-    '''
+    """
     summed_targets = {}
     for obj in objects:
         if isinstance(obj, SummedVariableUpdater):
@@ -253,22 +254,18 @@ def _check_multiple_summed_updaters(objects):
                     # We raise an error, even though this could be ok in
                     # principle (e.g. two Synapses could target different
                     # subsets of the target groups, without using subgroups)
-                    msg = ('Multiple "summed variables" target the '
-                           'variable "{var}" in group "{target}". Use '
-                           'multiple variables in the target group '
-                           'instead.'.format(var=obj.target_var.name,
-                                             target=obj.target.name))
+                    msg = (f"Multiple 'summed variables' target the "
+                           f"variable '{obj.target_var.name}' in group "
+                           f"'{obj.target.name}'. Use multiple variables in "
+                           f"the target group instead.")
                     raise NotImplementedError(msg)
                 elif (obj.target.start < other_target.stop and
                               other_target.start < obj.target.stop):
                     # Overlapping subgroups
-                    msg = ('Multiple "summed variables" target the '
-                           'variable "{var}" in overlapping groups '
-                           '"{target1}" and "{target2}". Use separate '
-                           'variables in the target groups '
-                           'instead.'.format(var=obj.target_var.name,
-                                             target1=other_target.name,
-                                             target2=obj.target.name))
+                    msg = (f"Multiple 'summed variables' target the "
+                           f"variable '{obj.target_var.name}' in overlapping "
+                           f"groups '{other_target.name}' and '{obj.target.name}'. "
+                           f"Use separate variables in the target groups instead.")
                     raise NotImplementedError(msg)
             summed_targets[obj.target_var] = obj.target
 
@@ -296,7 +293,7 @@ def _get_all_objects(objs):
 
 
 class Network(Nameable):
-    '''
+    """
     Network(*objs, name='network*')
     
     The main simulation controller in Brian
@@ -357,7 +354,7 @@ class Network(Nameable):
     --------
     
     MagicNetwork, run, stop
-    '''
+    """
 
     def __init__(self, *objs, **kwds):
         #: The set of objects in the Network, should not normally be modified
@@ -389,25 +386,25 @@ class Network(Nameable):
         self._schedule = None
      
     t = property(fget=lambda self: Quantity(self.t_, dim=second.dim, copy=False),
-                 doc='''
+                 doc="""
                      Current simulation time in seconds (`Quantity`)
-                     ''')
+                     """)
 
     @device_override('network_get_profiling_info')
     def get_profiling_info(self):
-        '''
+        """
         The only reason this is not directly implemented in `profiling_info`
         is to allow devices (e.g. `CPPStandaloneDevice`) to overwrite this.
-        '''
+        """
         if self._profiling_info is None:
-            raise ValueError('No profiling info collected (did you run with '
-                             'profile=True?)')
+            raise ValueError("No profiling info collected (did you run with "
+                             "'profile=True?')")
         return sorted(self._profiling_info, key=lambda item: item[1],
                       reverse=True)
 
     @property
     def profiling_info(self):
-        '''
+        """
         The time spent in executing the various `CodeObject` s.
 
         A list of ``(name, time)`` tuples, containing the name of the
@@ -417,43 +414,47 @@ class Network(Nameable):
 
         Profiling has to be activated using the ``profile`` keyword in `run` or
         `Network.run`.
-        '''
+        """
         return self.get_profiling_info()
 
     _globally_stopped = False
 
     def __getitem__(self, item):
         if not isinstance(item, str):
-            raise TypeError(('Need a name to access objects in a Network, '
-                             'got {type} instead').format(type=type(item)))
-        for obj in self.objects:
+            raise TypeError(f"Need a name to access objects in a Network, "
+                            f"got {type(item)} instead")
+        all_objects = _get_all_objects(self.objects)
+        for obj in all_objects:
             if obj.name == item:
                 return obj
 
-        raise KeyError('No object with name "%s" found' % item)
+        raise KeyError(f'No object with name "{item}" found')
 
     def __delitem__(self, key):
         if not isinstance(key, str):
-            raise TypeError(('Need a name to access objects in a Network, '
-                             'got {type} instead').format(type=type(key)))
+            raise TypeError("Need a name to access objects in a Network, "
+                            "got {type(key)} instead")
         for obj in self.objects:
             if obj.name == key:
                 self.remove(obj)
                 return
 
-        raise KeyError('No object with name "%s" found' % key)
+        raise KeyError(f"No object with name '{key}' found")
 
     def __contains__(self, item):
-        for obj in self.objects:
+        all_objects = _get_all_objects(self.objects)
+        for obj in all_objects:
             if obj.name == item:
                 return True
         return False
 
     def __len__(self):
-        return len(self.objects)
+        all_objects = _get_all_objects(self.objects)
+        return len(all_objects)
 
     def __iter__(self):
-        return iter(self.objects)
+        all_objects = _get_all_objects(self.objects)
+        return iter(all_objects)
 
     def add(self, *objs):
         """
@@ -472,12 +473,11 @@ class Network(Nameable):
         for obj in objs:
             if isinstance(obj, BrianObject):
                 if obj._network is not None:
-                    raise RuntimeError('%s has already been simulated, cannot '
-                                       'add it to the network. If you were '
-                                       'trying to remove and add an object to '
-                                       'temporarily stop it from being run, '
-                                       'set its active flag to False instead.'
-                                       % obj.name)
+                    raise RuntimeError(f"{obj.name} has already been simulated, cannot "
+                                       f"add it to the network. If you were "
+                                       f"trying to remove and add an object to "
+                                       f"temporarily stop it from being run, "
+                                       f"set its active flag to False instead.")
                 self.objects.add(obj)
             else:
                 # allow adding values from dictionaries
@@ -497,7 +497,7 @@ class Network(Nameable):
                                         "or containers of such objects to Network")
 
     def remove(self, *objs):
-        '''
+        """
         Remove an object or sequence of objects from a `Network`.
         
         Parameters
@@ -507,7 +507,7 @@ class Network(Nameable):
             The `BrianObject` or container of Brian objects to be removed. Specify
             multiple objects, or lists (or other containers) of objects.
             Containers will be removed recursively.
-        '''
+        """
         for obj in objs:
             if isinstance(obj, BrianObject):
                 self.objects.remove(obj)
@@ -536,7 +536,7 @@ class Network(Nameable):
 
     @device_override('network_store')
     def store(self, name='default', filename=None):
-        '''
+        """
         store(name='default', filename=None)
 
         Store the state of the network and all included objects.
@@ -560,7 +560,7 @@ class Network(Nameable):
         format of the file is not guaranteed to work across platforms or
         versions. If you are interested in storing the state of a network for
         documentation or analysis purposes use `Network.get_states` instead.
-        '''
+        """
         clocks = {obj.clock for obj in _get_all_objects(self.objects)}
         # Make sure that all clocks are up to date
         for clock in clocks:
@@ -589,7 +589,7 @@ class Network(Nameable):
     @device_override('network_restore')
     def restore(self, name='default', filename=None,
                 restore_random_state=False):
-        '''
+        """
         restore(name='default', filename=None, restore_random_state=False)
 
         Retore the state of the network and all included objects.
@@ -614,7 +614,7 @@ class Network(Nameable):
             that this also restores numpy's random number generator (since it is
             used internally by Brian), but it does *not* restore Python's
             builtin random number generator in the ``random`` module.
-        '''
+        """
         all_objects = _get_all_objects(self.objects)
         if filename is None:
             state = self._stored_state[name]
@@ -632,10 +632,10 @@ class Network(Nameable):
                 obj._restore_from_full_state(state[obj.name])
                 restored_objects.add(obj.name)
             elif hasattr(obj, '_restore_from_full_state'):
-                raise KeyError(('Stored state does not have a stored state for '
-                                '"%s". Note that the names of all objects have '
-                                'to be identical to the names when they were '
-                                'stored.') % obj.name)
+                raise KeyError(f"Stored state does not have a stored state for "
+                               f"'{obj.name}'. Note that the names of all objects have "
+                               f"to be identical to the names when they were "
+                               f"stored.")
         for clock in clocks:
             clock._restore_from_full_state(state[clock.name])
         clock_names = {c.name for c in clocks}
@@ -643,15 +643,15 @@ class Network(Nameable):
         unnused = (set(state.keys()) - restored_objects - clock_names -
                    {'0_t', '_random_generator_state'})
         if len(unnused):
-            raise KeyError('The stored state contains the state of the '
-                           'following objects which were not present in the '
-                           'network: %s. Note that the names of all objects '
-                           'have to be identical to the names when they were '
-                           'stored.' % (', '.join(unnused)))
+            raise KeyError(f"The stored state contains the state of the "
+                           f"following objects which were not present in the "
+                           f"network: {', '.join(unnused)}. Note that the names of all "
+                           f"objects have to be identical to the names when they "
+                           f"were stored.")
 
     def get_states(self, units=True, format='dict',
                    subexpressions=False, read_only_variables=True, level=0):
-        '''
+        """
         Return a copy of the current state variable values of objects in the
         network.. The returned arrays are copies of the actual arrays that
         store the state variable values, therefore changing the values in the
@@ -692,9 +692,9 @@ class Network(Nameable):
         See Also
         --------
         VariableOwner.get_states
-        '''
+        """
         states = dict()
-        for obj in self.objects:
+        for obj in self.sorted_objects:
             if hasattr(obj, 'get_states'):
                 states[obj.name] = obj.get_states(vars=None, units=units,
                                                   format=format,
@@ -704,7 +704,7 @@ class Network(Nameable):
         return states
 
     def set_states(self, values, units=True, format='dict', level=0):
-        '''
+        """
         Set the state variables of objects in the network.
 
         Parameters
@@ -723,7 +723,7 @@ class Network(Nameable):
         See Also
         --------
         Group.set_states
-        '''
+        """
         # For the moment, 'dict' is the only supported format -- later this will
         # be made into an extensible system, see github issue #306
         for obj_name, obj_values in values.items():
@@ -731,7 +731,7 @@ class Network(Nameable):
                 raise KeyError(("Network does not include a network with "
                                 "name '%s'.") % obj_name)
             self[obj_name].set_states(obj_values, units=units, format=format,
-                                     level=level+1)
+                                      level=level+1)
 
 
     def _get_schedule(self):
@@ -748,22 +748,21 @@ class Network(Nameable):
         else:
             if (not isinstance(schedule, Sequence) or
                     not all(isinstance(slot, str) for slot in schedule)):
-                raise TypeError('Schedule has to be None or a sequence of '
-                                'scheduling slots')
+                raise TypeError("Schedule has to be None or a sequence of "
+                                "scheduling slots")
             if any(slot.startswith('before_') or slot.startswith('after_')
                    for slot in schedule):
-                raise ValueError('Slot names are not allowed to start with '
-                                 '"before_" or "after_" -- such slot names '
-                                 'are created automatically based on the '
-                                 'existing slot names.')
+                raise ValueError("Slot names are not allowed to start with "
+                                 "'before_' or 'after_' -- such slot names "
+                                 "are created automatically based on the "
+                                 "existing slot names.")
             self._schedule = list(schedule)
-            logger.debug("Setting network {self.name} schedule to "
-                         "{self._schedule}".format(self=self),
+            logger.debug(f"Setting network '{self.name}' schedule to {self._schedule}",
                          "_set_schedule")
     
     schedule = property(fget=_get_schedule,
                         fset=_set_schedule,
-                        doc='''
+                        doc="""
         List of ``when`` slots in the order they will be updated, can be modified.
         
         See notes on scheduling in `Network`. Note that additional ``when``
@@ -773,11 +772,11 @@ class Network(Nameable):
 
         The schedule can also be set to ``None``, resetting it to the default
         schedule set by the `core.network.default_schedule` preference.
-        ''')
+        """)
 
     @property
     def sorted_objects(self):
-        '''
+        """
         The sorted objects of this network in the order defined by the schedule.
         
         Objects are sorted first by their ``when`` attribute, and secondly
@@ -790,23 +789,23 @@ class Network(Nameable):
 
         Final ties are resolved using the objects' names, leading to an
         arbitrary but deterministic sorting.
-        '''
+        """
         # Provided slot names are assigned positions 1, 4, 7, ...
         # before_... names are assigned positions 0, 3, 6, ...
         # after_... names are assigned positions 2, 5, 8, ...
         all_objects = _get_all_objects(self.objects)
         when_to_int = dict((when, 1+i*3)
                            for i, when in enumerate(self.schedule))
-        when_to_int.update(('before_' + when, i*3)
+        when_to_int.update((f"before_{when}", i*3)
                            for i, when in enumerate(self.schedule))
-        when_to_int.update(('after_' + when, 2+i*3)
+        when_to_int.update((f"after_{when}", 2+i*3)
                            for i, when in enumerate(self.schedule))
         return sorted(all_objects, key=lambda obj: (when_to_int[obj.when],
                                                     obj.order,
                                                     obj.name))
 
     def scheduling_summary(self):
-        '''
+        """
         Return a `SchedulingSummary` object, representing the scheduling
         information for all objects included in the network.
         
@@ -814,7 +813,7 @@ class Network(Nameable):
         -------
         summary : `SchedulingSummary`
             Object representing the scheduling information.
-        '''
+        """
         return SchedulingSummary(self.sorted_objects)
 
     def check_dependencies(self):
@@ -823,13 +822,13 @@ class Network(Nameable):
         for obj in all_objects:
             for dependency in obj._dependencies:
                 if not dependency in all_ids:
-                    raise ValueError(('"%s" has been included in the network '
-                                      'but not the object on which it '
-                                      'depends.') % obj.name)
+                    raise ValueError(f"'{obj.name}' has been included in the network "
+                                     f"but not the object on which it "
+                                     f"depends.")
 
     @device_override('network_before_run')
     def before_run(self, run_namespace):
-        '''
+        """
         before_run(namespace)
 
         Prepares the `Network` for a run.
@@ -842,7 +841,7 @@ class Network(Nameable):
         run_namespace : dict-like, optional
             A namespace in which objects which do not define their own
             namespace will be run.
-        '''
+        """
         all_objects = self.sorted_objects
         prefs.check_all_validated()
 
@@ -851,11 +850,11 @@ class Network(Nameable):
         non_unique_names = [name for name, count in Counter(names).items()
                             if count > 1]
         if len(non_unique_names):
-            formatted_names = ', '.join("'%s'" % name
+            formatted_names = ', '.join(f"'{name}'"
                                         for name in non_unique_names)
-            raise ValueError('All objects in a network need to have unique '
-                             'names, the following name(s) were used more than '
-                             'once: %s' % formatted_names)
+            raise ValueError(f"All objects in a network need to have unique "
+                             f"names, the following name(s) were used more than "
+                             f"once: {formatted_names}")
 
         # Check that there are no SummedVariableUpdaters targeting the same
         # target variable
@@ -870,24 +869,20 @@ class Network(Nameable):
             if device.network_schedule != self.schedule:
                 # TODO: The human-readable name of a device should be easier to get
                 device_name = list(all_devices.keys())[list(all_devices.values()).index(device)]
-                logger.warn(("The selected device '{device_name}' only "
-                             "supports a fixed schedule, but this schedule is "
-                             "not consistent with the network's schedule. The "
-                             "simulation will use the device's schedule.\n"
-                             "Device schedule: {device.network_schedule}\n"
-                             "Network schedule: {net.schedule}\n"
-                             "Set the network schedule explicitly or set the "
-                             "core.network.default_schedule preference to "
-                             "avoid this warning.").format(device_name=device_name,
-                                                           device=device,
-                                                           net=self),
+                logger.warn((f"The selected device '{device_name}' only "
+                             f"supports a fixed schedule, but this schedule is "
+                             f"not consistent with the network's schedule. The "
+                             f"simulation will use the device's schedule.\n"
+                             f"Device schedule: {device.network_schedule}\n"
+                             f"Network schedule: {self.schedule}\n"
+                             f"Set the network schedule explicitly or set the "
+                             f"core.network.default_schedule preference to "
+                             f"avoid this warning."),
                             name_suffix='schedule_conflict', once=True)
 
-        logger.debug("Preparing network {self.name} with {numobj} "
-                     "objects: {objnames}".format(self=self,
-                        numobj=len(all_objects),
-                        objnames=', '.join(obj.name for obj in all_objects)),
-                     "before_run")
+        objnames = ', '.join(obj.name for obj in all_objects)
+        logger.debug(f"Preparing network '{self.name}' with {len(all_objects)} "
+                     f"objects: {objnames}", "before_run")
 
         self.check_dependencies()
 
@@ -903,24 +898,22 @@ class Network(Nameable):
             if obj._network is None:
                 obj._network = self.id
             elif obj._network != self.id:
-                raise RuntimeError(('%s has already been run in the '
-                                    'context of another network. Use '
-                                    'add/remove to change the objects '
-                                    'in a simulated network instead of '
-                                    'creating a new one.') % obj.name)
+                raise RuntimeError(f"'{obj.name}' has already been run in the "
+                                   f"context of another network. Use "
+                                   f"add/remove to change the objects "
+                                   f"in a simulated network instead of "
+                                   f"creating a new one.")
 
-        logger.debug("Network {self.name} uses {num} "
-                     "clocks: {clocknames}".format(self=self,
-                        num=len(self._clocks),
-                        clocknames=', '.join('%s (dt=%s)' % (obj.name, obj.dt)
-                                             for obj in self._clocks)),
-                     "before_run")
+        clocknames = ', '.join(f"{obj.name} (dt={obj.dt})"
+                               for obj in self._clocks)
+        logger.debug(f"Network '{self.name}' uses {len(self._clocks)} "
+                     f"clocks: {clocknames}", "before_run")
 
     @device_override('network_after_run')
     def after_run(self):
-        '''
+        """
         after_run()
-        '''
+        """
         for obj in self.sorted_objects:
             if obj.active:
                 obj.after_run()
@@ -939,8 +932,8 @@ class Network(Nameable):
     @device_override('network_run')
     @check_units(duration=second, report_period=second)
     def run(self, duration, report=None, report_period=10*second,
-            namespace=None, profile=False, level=0):
-        '''
+            namespace=None, profile=None, level=0):
+        """
         run(duration, report=None, report_period=60*second, namespace=None, level=0)
         
         Runs the simulation for the given duration.
@@ -968,7 +961,8 @@ class Network(Nameable):
             and globals around the run function will be used.
         profile : bool, optional
             Whether to record profiling information (see
-            `Network.profiling_info`). Defaults to ``False``.
+            `Network.profiling_info`). Defaults to ``None`` (which will use the
+            value set by ``set_device``, if any).
         level : int, optional
             How deep to go up the stack frame to look for the locals/global
             (see `namespace` argument). Only used by run functions that call
@@ -979,9 +973,13 @@ class Network(Nameable):
         -----
         The simulation can be stopped by calling `Network.stop` or the
         global `stop` function.
-        '''
-        all_objects = self.sorted_objects
+        """
         device = get_device()  # Do not use the ProxyDevice -- slightly faster
+
+        if profile is None:
+            profile = device.build_options.get('profile', False)
+
+        all_objects = self.sorted_objects
         self._clocks = {obj.clock for obj in all_objects}
         single_clock = len(self._clocks) == 1
 
@@ -1012,9 +1010,7 @@ class Network(Nameable):
 
         start_time = time.time()
 
-        logger.debug("Simulating network '%s' from time %s to %s." % (self.name,
-                                                                      t_start,
-                                                                      t_end),
+        logger.debug(f"Simulating network '{self.name}' from time {t_start} to {t_end}.",
                      'run')
 
         if report is not None:
@@ -1025,15 +1021,14 @@ class Network(Nameable):
             elif report == 'stderr':
                 report_callback = TextReport(sys.stderr)
             elif isinstance(report, str):
-                raise ValueError(('Do not know how to handle report argument '
-                                  '"%s".' % report))
+                raise ValueError(f'Do not know how to handle report argument "{report}".')
             elif callable(report):
                 report_callback = report
             else:
-                raise TypeError(('Do not know how to handle report argument, '
-                                 'it has to be one of "text", "stdout", '
-                                 '"stderr", or a callable function/object, '
-                                 'but it is of type %s') % type(report))
+                raise TypeError(f"Do not know how to handle report argument, "
+                                f"it has to be one of 'text', 'stdout', "
+                                f"'stderr', or a callable function/object, "
+                                f"but it is of type {type(report)}")
             report_callback(0*second, 0.0, t_start, duration)
 
         profiling_info = defaultdict(float)
@@ -1126,35 +1121,34 @@ class Network(Nameable):
             report_callback((end_time-start_time)*second, 1.0, t_start, duration)
         self.after_run()
 
-        logger.debug(("Finished simulating network '%s' "
-                      "(took %.2fs)") % (self.name, end_time-start_time),
-                     'run')
+        logger.debug(f"Finished simulating network '{self.name}' "
+                     f"(took {end_time-start_time:.2f}s)", 'run')
         # Store profiling info (or erase old info to avoid confusion)
         if profile:
             self._profiling_info = [(name, t*second)
                                     for name, t in profiling_info.items()]
             # Dump a profiling summary to the log
-            logger.debug('\n' + str(profiling_summary(self)))
+            logger.debug(f"\n{str(profiling_summary(self))}")
         else:
             self._profiling_info = None
 
     @device_override('network_stop')
     def stop(self):
-        '''
+        """
         stop()
 
         Stops the network from running, this is reset the next time `Network.run` is called.
-        '''
+        """
         self._stopped = True
 
     def __repr__(self):
-        return '<%s at time t=%s, containing objects: %s>' % (self.__class__.__name__,
-                                                              str(self.t),
-                                                              ', '.join((obj.__repr__() for obj in _get_all_objects(self.objects))))
+        objects = ', '.join((obj.__repr__() for obj in _get_all_objects(self.objects)))
+        return (f"<{self.__class__.__name__} at time t={self.t!s}, containing "
+                f"objects: {objects}>")
 
 
 class ProfilingSummary(object):
-    '''
+    """
     Class to nicely display the results of profiling. Objects of this class are
     returned by `profiling_summary`.
 
@@ -1170,7 +1164,7 @@ class ProfilingSummary(object):
     See Also
     --------
     Network.profiling_info
-    '''
+    """
     def __init__(self, net, show=None):
         prof = net.profiling_info
         if len(prof):
@@ -1195,36 +1189,36 @@ class ProfilingSummary(object):
         self.times = times
 
     def __repr__(self):
-        times = ['%.2f %s' % (time/self.time_unit, self.time_unit) for time in self.times]
+        times = [f'{time / self.time_unit:.2f} {self.time_unit}' for time in self.times]
         times_maxlen = max(len(time) for time in times)
         times = [' '*(times_maxlen-len(time))+time for time in times]
-        percentages = ['%.2f %%' % percentage for percentage in self.percentages]
+        percentages = [f'{percentage:.2f} %' for percentage in self.percentages]
         percentages_maxlen = max(len(percentage) for percentage in percentages)
         percentages = [(' '*(percentages_maxlen-len(percentage)))+percentage for percentage in percentages]
 
         s = 'Profiling summary'
-        s += '\n'+'='*len(s)+'\n'
+        s += f"\n{'=' * len(s)}\n"
         for name, time, percentage in zip(self.names, times, percentages):
-            s += '%s    %s    %s\n' % (name, time, percentage)
+            s += f'{name}    {time}    {percentage}\n'
         return s
 
     def _repr_html_(self):
-        times = ['%.2f %s' % (time/self.time_unit, self.time_unit) for time in self.times]
-        percentages = ['%.2f %%' % percentage for percentage in self.percentages]
+        times = [f'{time / self.time_unit:.2f} {self.time_unit}' for time in self.times]
+        percentages = [f'{percentage:.2f} %' for percentage in self.percentages]
         s = '<h2 class="brian_prof_summary_header">Profiling summary</h2>\n'
         s += '<table class="brian_prof_summary_table">\n'
         for name, time, percentage in zip(self.names, times, percentages):
             s += '<tr>'
-            s += '<td>%s</td>' % name
-            s += '<td style="text-align: right">%s</td>' % time
-            s += '<td style="text-align: right">%s</td>' % percentage
+            s += f'<td>{name}</td>'
+            s += f'<td style="text-align: right">{time}</td>'
+            s += f'<td style="text-align: right">{percentage}</td>'
             s += '</tr>\n'
         s += '</table>'
         return s
 
 
 def profiling_summary(net=None, show=None):
-    '''
+    """
     Returns a `ProfilingSummary` of the profiling info for a run. This object
     can be transformed to a string explicitly but on an interactive console
     simply calling `profiling_summary` is enough since it will
@@ -1238,7 +1232,7 @@ def profiling_summary(net=None, show=None):
     show : int
         The number of results to show (the longest results will be shown). If
         not specified, all results will be shown.
-    '''
+    """
     if net is None:
         from .magic import magic_network
         net = magic_network
@@ -1246,7 +1240,7 @@ def profiling_summary(net=None, show=None):
 
 
 def scheduling_summary(net=None):
-    '''
+    """
     Returns a `SchedulingSummary` object, representing the scheduling
     information for all objects included in the given `Network` (or the
     "magic" network, if none is specified). The returned objects can be
@@ -1264,7 +1258,7 @@ def scheduling_summary(net=None):
     -------
     summary : `SchedulingSummary`
         An object that represents the scheduling information.
-    '''
+    """
     if net is None:
         from .magic import magic_network
         magic_network._update_magic_objects(level=1)
@@ -1273,7 +1267,7 @@ def scheduling_summary(net=None):
 
 
 def schedule_propagation_offset(net=None):
-    '''
+    """
     Returns the minimal time difference for a post-synaptic effect after a
     spike. With the default schedule, this time difference is 0, since the
     ``thresholds`` slot precedes the ``synapses`` slot. For the GeNN device,
@@ -1295,7 +1289,7 @@ def schedule_propagation_offset(net=None):
     -----
     This function always returns ``0*ms`` or ``defaultclock.dt`` -- no attempt
     is made to deal with other clocks.
-    '''
+    """
     from brian2.core.magic import magic_network
 
     device = get_device()
